@@ -1,15 +1,15 @@
 package Net::DNS::Parameters;
 
 #
-# $Id: Parameters.pm 1527 2017-01-18 21:42:48Z willem $
+# $Id: Parameters.pm 1543 2017-02-28 19:27:23Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1543 $)[1];
 
 
 ################################################
 ##
 ##	Domain Name System (DNS) Parameters
-##	(last updated 2016-12-09)
+##	(last updated 2017-02-23)
 ##
 ################################################
 
@@ -19,17 +19,6 @@ use warnings;
 use integer;
 use Carp;
 
-
-our $DNSEXTLANG = 'ARPA.';		## draft-levine-dnsextlang
-
-use constant DNSEXTLANG => defined eval <<'END';
-	die 'preempt failure' if $^O =~ /cygwin|MSWin32/i;
-	require IO::File;
-	local $SIG{__WARN__} = sub { };
-	new IO::File('RRTYPEgen |');
-END
-
-
 use base qw(Exporter);
 our @EXPORT = qw(
 		classbyname classbyval %classbyname
@@ -197,6 +186,7 @@ our %ednsoptionbyname = (
 	'TCP-KEEPALIVE' => 11,					# RFC7828
 	PADDING		=> 12,					# RFC7830
 	CHAIN		=> 13,					# RFC7901
+	'KEY-TAG'	=> 14,					# RFC-ietf-dnsop-edns-key-tag-05
 	DEVICEID	=> 26946,				# https://docs.umbrella.com/developer/networkdevices-api/identifying-dns-traffic2
 	);
 our %ednsoptionbyval = reverse %ednsoptionbyname;
@@ -314,6 +304,16 @@ sub ednsoptionbyval {
 }
 
 
+our $DNSEXTLANG = 'ARPA.';		## draft-levine-dnsextlang
+
+use constant DNSEXTLANG => defined eval <<'END';
+	die 'preempt failure' if $^O =~ /cygwin|MSWin32/i;
+	require IO::File;
+	local $SIG{__WARN__} = sub { };
+	new IO::File('RRTYPEgen |') or die $!;
+END
+
+
 sub register {				## register( 'TOY', 1234 )	(NOT part of published API)
 	my ( $mnemonic, $rrtype ) = map uc($_), @_;		# uncoverable pod
 	$rrtype = rand(255) + 65280 unless $rrtype;
