@@ -1,10 +1,9 @@
 package Net::DNS::Resolver;
 
 #
-# $Id: Resolver.pm 1475 2016-04-15 09:46:40Z willem $
+# $Id: Resolver.pm 1527 2017-01-18 21:42:48Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1475 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1527 $)[1];
 
 =head1 NAME
 
@@ -14,6 +13,7 @@ Net::DNS::Resolver - DNS resolver class
 
 
 use strict;
+use warnings;
 
 use constant CONFIG => defined eval "require Net::DNS::Resolver::$^O";
 
@@ -206,6 +206,8 @@ If the name looks like an IP address (IPv4 or IPv6),
 then a query within in-addr.arpa or ip6.arpa will be performed.
 
 Returns a C<Net::DNS::Packet> object, or C<undef> if no answers were found.
+The reason for failure may be determined using errorstring().
+
 If you need to examine the response packet, whether it contains
 any answers or not, use the send() method instead.
 
@@ -243,6 +245,8 @@ If the name looks like an IP address (IPv4 or IPv6),
 then a query within in-addr.arpa or ip6.arpa will be performed.
 
 Returns a C<Net::DNS::Packet> object, or C<undef> if no answers were found.
+The reason for failure may be determined using errorstring().
+
 If you need to examine the response packet, whether it contains
 any answers or not, use the send() method instead.
 
@@ -295,6 +299,11 @@ When called in list context, axfr() returns a list of C<Net::DNS::RR>
 objects.  The redundant SOA record that terminates the zone transfer
 is not returned to the caller.
 
+In deferrence to RFC1035(6.3), a complete zone transfer is expected
+to return all records in the zone or nothing at all.
+When no resource records are returned by axfr(),
+the reason for failure may be determined using errorstring().
+
 Here is an example that uses a timeout and TSIG verification:
 
     $resolver->tcp_timeout( 10 );
@@ -305,14 +314,13 @@ Here is an example that uses a timeout and TSIG verification:
 	$rr->print;
     }
 
-In deferrence to RFC1035(6.3), a complete zone transfer is expected
-to return all records in the zone or nothing at all.
-An exception is raised if the zone transfer can not be completed.
-
 
 When called in scalar context, axfr() returns an iterator object.
 Each invocation of the iterator returns a single C<Net::DNS::RR>
 or C<undef> when the zone is exhausted.
+
+An exception is raised if the zone transfer can not be completed.
+
 The redundant SOA record that terminates the zone transfer is not
 returned to the caller.
 
@@ -347,8 +355,8 @@ then a query within in-addr.arpa or ip6.arpa will be performed.
 
 Returns an opaque handle which is passed to subsequent invocations of
 the C<bgbusy> and C<bgread> methods.
-Errors are indicated by returning C<undef> in which case the
-reason for failure can be found by calling the errorstring method.
+Errors are indicated by returning C<undef> in which case
+the reason for failure may be determined using errorstring().
 
 The program may determine when the handle is ready for reading by
 calling C<bgbusy>.
@@ -604,7 +612,8 @@ response to a query.
 
     print 'query status: ', $resolver->errorstring, "\n";
 
-Returns a string containing the status of the most recent query.
+Returns a string containing error information from the most recent method call.
+errorstring() is meaningful only when interrogated immediately after an error.
 
 
 =head2 dnssec
@@ -738,10 +747,10 @@ take values are specified as C<option:value>.
 =head1 IPv6 TRANSPORT
 
 The Net::DNS::Resolver library will enable IPv6 transport if the
-appropriate libraries (IO::Socket::INET6 or IO::Socket::IP) are
+appropriate libraries (IO::Socket::IP or IO::Socket::INET6) are
 available and the destination nameserver has an IPv6 address.
 
-The force_v4(), force_v6(),prefer_v4 and prefer_v6() methods with a
+The force_v4(), force_v6(), prefer_v4 and prefer_v6() methods with a
 non-zero argument may be used to configure transport selection.
 
 The behaviour of the nameserver() method illustrates the transport
@@ -814,7 +823,7 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl>, L<Net::DNS>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
 L<Net::DNS::Header>, L<Net::DNS::Question>, L<Net::DNS::RR>,
-L<resolver(5)>, RFC 1035, RFC 1034 Section 4.3.5
+L<resolver(5)>, RFC 1034, RFC 1035
 
 =cut
 
